
# q-expansion of newform 555.2.e.b, downloaded from the LMFDB on 03 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 555
weight = 2
poly_data = [5, -24, 67, -40, 18, -24, 17, 2, 2, -4, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[5989545, -291116, 1298629, -3171794, 1157496, 1133738, 361520, -499512, -8715, 37174], 2155475], [[-889465, -483518, 1030392, 941963, -232917, -1448901, -95540, 268424, 97580, -48698], 2155475], [[-1119165, 5550587, -3356378, 1513733, -598972, 1300359, -87740, 11334, -209545, 66007], 2155475], [[-7025855, -948521, 2057749, 1658061, -558524, -2434097, -273780, 488178, 218260, -103181], 2155475], [[-1071605, -2887661, 7675534, -2738924, 728141, -2873002, -234805, 46398, 569660, -172521], 2155475], [[-44845, 217231, -136214, 43254, -88486, 63092, 12155, 5217, -14385, 3641], 39550], [[-612945, 3175977, -1009448, 1604588, -1253092, 1178554, 172645, 32639, -277585, 80807], 862190], [[13619765, -66097717, 42063298, -12924328, 21919152, -18169694, -2135635, -1597919, 3960845, -1014787], 4310950], [[-8159055, 39767679, -24512876, 8734036, -13586024, 11194578, 1504470, 820553, -2441740, 637619], 2155475]]

hecke_ring_character_values = [[371, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [112, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [76, [-1, 0, 0, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 1, 0, 0, 0, 0, 0, 0], [1, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 1, 0, 0, 0], [2, 0, 0, 0, 1, 0, 0, 0, 0, 0], [0, 0, 1, 0, -1, 0, 0, 0, 0, 0], [0, 0, 0, 1, 0, 0, 1, -1, 1, 0], [0, 0, 0, -1, 0, 0, -2, 0, -2, 0], [0, 0, 0, 1, 0, 0, -1, -1, 0, 1], [0, 0, 0, 0, 0, 0, 1, 1, 2, 2], [0, 0, 0, -1, 0, 0, -2, 1, 0, 0], [0, 0, 0, 1, 0, 0, 2, -1, -1, -3], [0, -1, 2, 1, 0, 1, 1, 0, 2, 0], [1, -1, 1, 0, -2, -2, 0, 0, 0, 0], [0, 0, 0, 1, 0, 0, -3, 0, 0, -1], [-2, -1, -1, 0, -1, 2, 0, 0, 0, 0], [-1, -1, -5, 0, 1, -1, 0, 0, 0, 0], [0, 0, 0, -4, 0, 0, -3, 1, -1, 2], [0, 0, 0, -1, 0, 0, 4, -1, 1, 1], [2, -3, 0, 0, 0, 1, 0, 0, 0, 0], [-1, -1, -4, 0, 1, 0, 0, 0, 0, 0], [2, -1, -2, 0, -3, 1, 0, 0, 0, 0], [0, 0, 0, -2, 0, 0, 1, 0, -1, -4], [-2, 1, 2, 0, -1, -2, 0, 0, 0, 0], [0, 0, 0, 2, 0, 0, 2, -1, 2, -3], [0, 0, 0, -6, 0, 0, -6, 0, -2, 3], [1, 3, -4, 0, 3, 0, 0, 0, 0, 0], [0, 0, 0, -1, 0, 0, -3, -1, 1, -2], [3, -3, -1, 0, 3, 3, 0, 0, 0, 0], [0, 0, 0, 1, 0, 0, 7, -1, 4, 1], [0, 0, 0, -1, 0, 0, 3, -1, 2, -2], [4, -1, -4, 0, 3, 1, 0, 0, 0, 0], [0, 0, 0, 1, 0, 0, 3, -2, 3, 2], [-1, -1, 4, 0, 1, 5, 0, 0, 0, 0], [2, -3, -4, 0, -2, -4, 0, 0, 0, 0], [7, -3, -3, 0, -2, -4, 0, 0, 0, 0], [-1, -4, 0, 0, 0, 2, 0, 0, 0, 0], [-4, 0, 2, 0, -5, 0, 0, 0, 0, 0], [0, 0, 0, 2, 0, 0, -2, -2, 8, 2], [0, 0, 0, -7, 0, 0, 2, -2, -4, 2], [4, 3, -3, 0, 5, 2, 0, 0, 0, 0], [0, 0, 0, -6, 0, 0, 10, -1, 0, -1], [0, 2, 5, 0, 1, -1, 0, 0, 0, 0], [0, 0, 0, 2, 0, 0, 0, 0, 0, -8], [0, 0, 0, -1, 0, 0, -11, 5, -3, 5], [-7, -2, 6, 0, -2, -1, 0, 0, 0, 0], [0, 0, 0, 3, 0, 0, 12, 1, 1, -3], [-8, 2, 7, 0, -3, 1, 0, 0, 0, 0], [2, 0, -6, 0, 3, 2, 0, 0, 0, 0], [0, 0, 0, -4, 0, 0, -4, 4, -2, 6], [-5, 3, -1, 0, -1, -3, 0, 0, 0, 0], [-3, 4, 0, 0, 0, 1, 0, 0, 0, 0], [0, 0, 0, -6, 0, 0, -10, 3, -4, 5], [0, 0, 0, 4, 0, 0, -7, 6, -9, 0], [0, 0, 0, 1, 0, 0, -16, -1, 2, 2], [0, 0, 0, 3, 0, 0, 8, 0, -4, -4], [-1, -5, -2, 0, -5, -1, 0, 0, 0, 0], [-5, 1, -5, 0, 6, 2, 0, 0, 0, 0], [5, 1, 7, 0, -1, 1, 0, 0, 0, 0], [0, 0, 0, 4, 0, 0, -6, 2, -6, 0], [0, 0, 0, 3, 0, 0, -4, 5, -6, -2], [0, 0, 0, -2, 0, 0, 6, 5, -1, 0], [-3, 2, -2, 0, 0, -5, 0, 0, 0, 0], [0, 0, -10, 0, -1, -2, 0, 0, 0, 0], [0, 0, 0, -1, 0, 0, -5, 3, -1, -1], [0, 0, 0, 13, 0, 0, 3, 4, 2, -3], [-13, 7, 7, 0, -3, 1, 0, 0, 0, 0], [0, 0, 0, -9, 0, 0, 9, 1, -4, 5], [0, 4, 6, 0, 1, 4, 0, 0, 0, 0], [0, 0, 0, -8, 0, 0, 10, 0, -4, -4], [-4, -1, 8, 0, -12, -6, 0, 0, 0, 0], [0, 0, 0, 6, 0, 0, 11, 5, 0, 2], [-11, -7, 7, 0, 0, 0, 0, 0, 0, 0], [0, 3, -2, 0, -4, -5, 0, 0, 0, 0], [2, 1, -2, 0, -2, 3, 0, 0, 0, 0], [12, 1, 6, 0, 8, 0, 0, 0, 0, 0], [0, 0, 0, 3, 0, 0, -11, -5, 6, -2], [0, 0, 0, 1, 0, 0, 13, 5, 1, -1], [12, 4, -2, 0, 7, 2, 0, 0, 0, 0], [0, 0, 0, 9, 0, 0, 5, -3, 1, 1], [0, 0, 0, 13, 0, 0, 9, 3, -6, 1], [10, -4, 0, 0, 6, 8, 0, 0, 0, 0], [0, 0, 0, 1, 0, 0, 18, 1, 3, -3], [0, 0, 0, 0, 0, 0, -10, -4, 2, 0], [10, 5, 8, 0, 0, 1, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 12, -2, -2, 0], [4, 5, -3, 0, 1, -4, 0, 0, 0, 0], [0, 0, 0, -9, 0, 0, 0, -6, -2, 3], [0, 0, 0, 13, 0, 0, -9, 3, -3, -3], [0, 0, 0, 2, 0, 0, -6, -1, -6, -1], [0, 0, 0, -3, 0, 0, -13, 1, -1, 9], [0, 0, 0, -1, 0, 0, 1, -5, 2, -2], [0, 0, 0, -8, 0, 0, 8, 1, -6, 3], [0, 0, 0, 3, 0, 0, -11, 0, 10, 4], [-3, 5, 1, 0, -4, -4, 0, 0, 0, 0], [0, 0, 0, -2, 0, 0, 0, -2, -10, -10], [0, 0, 0, -2, 0, 0, 2, -8, 0, 0], [6, 2, -4, 0, 8, 8, 0, 0, 0, 0], [1, 3, 10, 0, 5, 0, 0, 0, 0, 0], [0, 0, 0, 4, 0, 0, -4, -5, 5, 3], [0, 0, 0, 7, 0, 0, -14, -1, -3, -9], [0, 0, 0, 9, 0, 0, -7, -1, 11, -3], [0, 0, 0, -11, 0, 0, -9, 3, 2, 6], [0, 0, 0, 1, 0, 0, 0, 4, -2, -10], [0, 0, 0, 10, 0, 0, -5, 4, -7, 3], [4, -5, -5, 0, 9, 5, 0, 0, 0, 0], [0, 0, 0, 8, 0, 0, -6, 5, 1, 8], [0, 0, 0, 12, 0, 0, 3, 1, 4, 6], [4, -5, -15, 0, -5, -6, 0, 0, 0, 0], [5, -9, -3, 0, 8, 4, 0, 0, 0, 0], [-13, 3, 5, 0, 1, 7, 0, 0, 0, 0], [0, 0, 0, 2, 0, 0, -12, 0, 6, -1], [-10, 6, -2, 0, 2, 2, 0, 0, 0, 0], [-17, 6, 7, 0, -8, -7, 0, 0, 0, 0], [-4, -10, 0, 0, -2, 2, 0, 0, 0, 0], [0, 0, 0, -14, 0, 0, -3, 0, -11, 4], [-7, 5, 11, 0, -4, -2, 0, 0, 0, 0], [0, 0, 0, 5, 0, 0, 25, -8, 2, -8], [0, 0, 0, 8, 0, 0, 16, -4, 4, -10], [0, 0, 0, -10, 0, 0, -14, 6, 0, 12], [7, -3, 21, 0, -2, 4, 0, 0, 0, 0], [0, 0, 0, 5, 0, 0, 12, 5, -5, 3], [0, 5, -2, 0, -2, 1, 0, 0, 0, 0], [15, 3, 2, 0, 5, 5, 0, 0, 0, 0], [0, 0, 0, 10, 0, 0, -4, -4, 4, 2], [5, -10, 0, 0, 2, 4, 0, 0, 0, 0], [0, 0, 0, 3, 0, 0, 11, -7, -7, -15], [0, 0, 0, -8, 0, 0, -18, 4, 0, 0], [-1, 1, -5, 0, -2, 0, 0, 0, 0, 0], [0, 0, 0, -2, 0, 0, 0, -5, 7, 4], [-10, -2, 4, 0, 5, -2, 0, 0, 0, 0], [-18, 1, 10, 0, -4, -4, 0, 0, 0, 0], [3, 4, 5, 0, 4, 5, 0, 0, 0, 0], [15, -5, 8, 0, -4, 0, 0, 0, 0, 0], [0, 0, 0, -9, 0, 0, -9, -6, 4, 5], [16, 4, 3, 0, 13, 2, 0, 0, 0, 0], [0, 0, 0, 3, 0, 0, 1, 5, 8, -9], [-14, 8, 2, 0, -10, 0, 0, 0, 0, 0], [8, -9, 2, 0, -13, -1, 0, 0, 0, 0], [0, 0, 0, 10, 0, 0, 23, -7, 0, -2], [0, 0, 0, -10, 0, 0, 3, -1, -7, 0], [11, -10, -9, 0, 9, -3, 0, 0, 0, 0], [-19, 7, 5, 0, -16, -4, 0, 0, 0, 0], [4, -12, 2, 0, 0, 2, 0, 0, 0, 0], [0, 0, 0, 6, 0, 0, -23, 5, -2, -6], [0, 0, 0, 8, 0, 0, 2, 6, 6, 8], [9, 7, -7, 0, 14, 6, 0, 0, 0, 0], [0, 0, 0, -5, 0, 0, 17, -1, -9, -5], [0, 0, 0, 18, 0, 0, -11, 3, -2, -2], [0, 0, 0, -6, 0, 0, 6, -4, 0, 6], [24, 1, 0, 0, 5, 4, 0, 0, 0, 0], [8, -17, -6, 0, 1, 7, 0, 0, 0, 0], [5, 3, 26, 0, -1, 2, 0, 0, 0, 0], [0, 0, 0, -20, 0, 0, 12, 0, -6, -5], [-11, 10, 0, 0, -2, -7, 0, 0, 0, 0], [0, 0, 0, -7, 0, 0, -21, -3, -1, -9], [0, 0, 0, -6, 0, 0, 2, -1, 2, 17], [0, 0, 0, -10, 0, 0, -12, -2, -2, 10], [0, 0, -10, 0, -10, -6, 0, 0, 0, 0], [-6, -1, 4, 0, -10, 7, 0, 0, 0, 0], [2, 2, 3, 0, -1, 2, 0, 0, 0, 0], [0, 0, 0, 12, 0, 0, 22, 6, 0, 2], [-24, -4, -6, 0, 6, 2, 0, 0, 0, 0], [0, 0, 0, 2, 0, 0, 10, 3, 7, 3], [15, -9, -11, 0, 10, -4, 0, 0, 0, 0], [0, 0, 0, -21, 0, 0, -13, 1, -4, 8], [-10, -12, 5, 0, -6, -2, 0, 0, 0, 0], [0, 0, 0, -14, 0, 0, -23, -8, -3, -8], [0, 0, 0, -4, 0, 0, 2, 5, -13, -7]]
